// Copyright 1999-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import <OmniBase/rcsid.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>

#include "CCArgs.h"
#include "CCErrno.h"
#include "CCMckLib.h"

#include "CCConfig.h"
#include "CCMckDirect.h"
#include "CCMerchantTest.h"
#include "CCMerchantCustom.h"

RCS_ID("$Header: /Network/Source/CVS/OmniGroup/Frameworks/OmniCyberCash/Server/omni-server.c,v 1.6 2000/01/19 23:36:27 kc Exp $")

/* You'll want to customize this. */
const char *myConfigFile = "/install-mck-3_2_0_4-linux_0/mck-3.2.0.4-linux/merchant-stuff/conf/merchant_conf";

#define LISTEN_PORT 5746
#define MAX_CONNECTIONS 20

int main(int argc, char *argv[])
{
    CCNVlistPtr args = NULL;
    CCNVlistPtr result = NULL;
    int i, listening, connections[MAX_CONNECTIONS], max;
    struct sockaddr_in address;
    fd_set readSet, exceptionSet;

    for (i = 0; i < MAX_CONNECTIONS; i++)
        connections[i] = -1;

    /* set program name global in libmck */
    progname = argv[0];

    /* read the configuration into the Global Config record in libmck */
    CCMckDirectSetup (myConfigFile);

    listening = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    address.sin_family = AF_INET;
    address.sin_addr.s_addr = htonl(INADDR_ANY);
    address.sin_port = htons(LISTEN_PORT);
    if (bind(listening, &address, sizeof(address)) == -1) {
        printf("couldn't bind\n");
        exit(1);
    }
    if (listen(listening, 1) == -1) {
        printf("couldn't listen\n");
        exit(1);
    }
    printf("listening...\n");
    while(1) {
        int length;
        char message[100];
        char parameters[10000];
        CCNVlistPtr args, result;
        char *resultString;

        max = -1;
        for (i = 0; i < MAX_CONNECTIONS; i++) {
            if (connections[i] != -1) {
                FD_SET(readSet, connections[i]);
                FD_SET(exceptionSet, connections[i]);
                if (connections[i] > max)
                    max = connections[i];
            }
        }
        FD_SET(readSet, listening);
        if (listening > max)
            max = listening;
        select(max, &readSet, NULL, &exceptionSet, NULL);

        /* remove any connections that died */
        for (i = 0; i < MAX_CONNECTIONS; i++) {
            if (FD_ISSET(connections[i], exceptionSet)) {
                close(connections[i]);
                printf("connection closed %d\n", i);
                connections[i] = -1;
            }
        }

        /* accept a new connection, if any are waiting */
        if (FD_ISSET(listening, readSet)) {
            for (i = 0; i < MAX_CONNECTIONS; i++) {
                if (connections[i] == -1) {
                    connections[i] = accept(listening, &address, &length);
                    printf("accepted connection %d\n", i);
                }
            }
        }

        /* handle a message */
        for (i = 0; i < MAX_CONNECTIONS; i++) {
            if (FD_ISSET(connections[i], readSet)) {
                CCSocketReadLine(connection, message, 100);
                if (!strlen(message))
                    break;
                message[strlen(message)-1] = '\0';
                CCSocketReadLine(connection, parameters, 10000);
                parameters[strlen(parameters)-1] = '\0';
                printf("read message: %s params: %s\n", message, parameters);

                args = CCUrlDecode(parameters);
                printf("decoded to:\n");
                CCDumpNVlist(stdout, args);
                result = SendCC2_1Server(message, args);
                resultString = CCUrlEncode(result);
                printf("got result from CC: %s\n", resultString);
                CCSocketWrite(connection, resultString);
                CCSocketWrite(connection, "\n");

                free(resultString);
                CCDeleteNVlist(args);
                CCDeleteNVlist(result);
                printf("cleanup complete\n");
                break;
            }
        }
    }
}
